SystemOrganization addCategory: #'GMF-MetaModel'!
SystemOrganization addCategory: #'GMF-MetaModel-Tests'!

TestCase subclass: #TestMetaModel
	instanceVariableNames: 'metaModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel-Tests'!

!TestMetaModel methodsFor: 'initialize-release' stamp: 'volosmat 3/18/2014 22:27'!
setUp
	
	self
		setUpDB! !

!TestMetaModel methodsFor: 'private' stamp: 'volosmat 3/18/2014 22:26'!
setUpDB
	
	metaModel := MetaModel new! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/3/2014 14:41'!
testAddAssociationToElement
	| start end |
	start := Quantity name: 'Minerals' metaModel: metaModel.
	end := Quantity name: 'MineralWater' metaModel: metaModel.
				
	SubQuantityOf
		name: 'quantity'
		start: start
		startMultiplicity: '1'
		end: end
		endMultiplicity: '1'.
		
	self assert: (start searchAssociation: 'quantity') notNil & (end searchAssociation: 'quantity') notNil ! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/3/2014 14:15'!
testAddAttributeToElement
	| element |
	element := Kind name: 'ElementWithAttr' metaModel: metaModel.
	Attribute name: 'TestAttr' dataType: 'string'  multiplicity: '0'  element: element.
	
	self assert: ( element searchAttribute: 'TestAttr' ) notNil.
	! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/3/2014 12:34'!
testAddElement
	"relation between element and metamodel is inverse is instance of element is created, element is automatically added to metamodel collection"
	Kind name: 'TestElement' metaModel: metaModel.
	
	self assert: ( metaModel modelElements anySatisfy: [ :each | each name = 'TestElement' ])! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/3/2014 14:40'!
testAddMethodToElement
	| element |
	element := Kind name: 'ElementWithMethod' metaModel: metaModel.
	Method name: 'TestMethod' element: element.

	self assert: ( element searchMethod: 'TestMethod' ) notNil
	! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/3/2014 21:53'!
testAddNonUniqueElement

	Kind name: 'TestElement' metaModel: metaModel. 
	Phase name: 'TestElementPhase' metaModel: metaModel.
	Role name: 'TestElement' metaModel: metaModel.
	
	self assert: (metaModel modelElements
			detect: [:each | each isMemberOf: Kind]) notNil & metaModel modelElements size = 2! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/4/2014 15:47'!
testInverseRelationElementAssociation
	| start end |
	FormalAssociation 
		name: 'containment'
		start: (start := Kind name: 'Bottle' metaModel: metaModel) 
		startMultiplicity: '1'
		end: (end := Quantity name: 'MineralWater' metaModel: metaModel)
		endMultiplicity: '1'.
	
	self assert: (( start searchAssociation: 'containment' ) name = 'containment').
	self assert: (( end searchAssociation: 'containment' ) name = 'containment')
	! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/4/2014 15:35'!
testInverseRelationElementAttribute
	| element |
	element := Kind name: 'TestElement' metaModel: metaModel. 
	Attribute
		name: 'Attribute'
		dataType: 'string'
		multiplicity: '1'
		element: element.
		
	self assert: (element searchAttribute: 'Attribute') name = 'Attribute'! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/4/2014 15:37'!
testInverseRelationElementMethod
	| element |
	element := Kind name: 'TestElement' metaModel: metaModel. 
	Method name: 'Method' element: element. 
		
	self assert: (element searchMethod: 'Method') name = 'Method'
	
	
	! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/4/2014 18:16'!
testInverseRelationElementParent
	| element |
	element := SubKind name: 'SubKindTestElement' metaModel: metaModel.
	
	ParentClass parentClass: (Kind name: 'TestChildKind' metaModel: metaModel) child: element.
	 
	self assert: (element searchParent: 'TestChildKind') parentClass name = 'TestChildKind'
	! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/27/2014 13:32'!
testInverseRelationMetaModelElement
	| element |
	element := Kind name: 'TestInverseKind' metaModel: metaModel.
	
	self assert: (metaModel modelElements at: element name) name = 'TestInverseKind'! !

!TestMetaModel methodsFor: 'running' stamp: 'volosmat 4/27/2014 13:31'!
testRemoveElement
	| objectToRemove |
	
	objectToRemove := Kind name: 'ElementToRemove' metaModel: metaModel.
	
	metaModel removeElement: objectToRemove.
	
	self assert: metaModel modelElements size = 0
	
	
	
	! !

TestCase subclass: #TestOntoUMLAssociationRules
	instanceVariableNames: 'metaModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel-Tests'!

!TestOntoUMLAssociationRules methodsFor: 'initialize-release' stamp: 'volosmat 4/3/2014 20:49'!
setUp
	
	self setUpDB! !

!TestOntoUMLAssociationRules methodsFor: 'private' stamp: 'volosmat 4/3/2014 20:49'!
setUpDB
	metaModel := MetaModel new! !

!TestOntoUMLAssociationRules methodsFor: 'running' stamp: 'volosmat 4/25/2014 19:26'!
testComponentOf
	| brain person component |
	person 	:= Kind name: 'Person' metaModel: metaModel.
	brain    	:= Kind name: 'Brain' metaModel: metaModel.
	
	component := ComponentOf 	name: 				'brain' 
									start:				brain
									startMultiplicity: 	'1'
									end: 				person
									endMultiplicity: 		'1'.
	self assert: component isElementValid.
	
	component := ComponentOf 	name: 				'heart' 
									start:				(Phase name: 'heart' metaModel: metaModel)
									startMultiplicity: 	'1'
									end: 				person
									endMultiplicity: 		'1'.
	self deny: component isElementValid.
! !

!TestOntoUMLAssociationRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 21:33'!
testDerivation
	| derivation |
	derivation := Derivation
				name: 'Derivation'
				start: (Relator name: 'Relator' metaModel: metaModel) 
				startMultiplicity: '1'
				end: (MaterialAssociation
						name: 'MaterialAssociation'
						start: (Role name: 'Role' metaModel: metaModel)
						end: (Role name: 'Role2' metaModel: metaModel))
				endMultiplicity: '1..*'.
	self assert: derivation isElementValid.
	
	derivation endMultiplicity: '0..n'.
	
	self deny: derivation isElementValid
	! !

!TestOntoUMLAssociationRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 21:47'!
testMaterialAssociation
	| materialAssociation |
	materialAssociation := MaterialAssociation
				name: 'Material'
				start: (Role name: 'Student' metaModel: metaModel)
				startMultiplicity: '0..*'
				end: (Role name: 'School' metaModel: metaModel)
				endMultiplicity: '1..n'.
				
	self deny: materialAssociation isElementValid.
	
	materialAssociation startMultiplicity: '1..*'.
	Derivation
		name: 'Derivation'
		start: (Relator name: 'Relator' metaModel: metaModel)
		startMultiplicity: '1'
		end: materialAssociation
		endMultiplicity: '1..*'.
	
	self assert: materialAssociation isElementValid! !

!TestOntoUMLAssociationRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 21:17'!
testMediation
	| mediation |
	mediation := Mediation name: 'Mediation' start: ( Relator name: 'Relator' metaModel: metaModel )  startMultiplicity: '1..n' end: ( Role name: 'Role1' metaModel: metaModel )  endMultiplicity: '1'.
	
	self assert: mediation isElementValid.
	
	mediation startMultiplicity: '0..n'.
	mediation endMultiplicity: '0..n'.
	
	self deny: mediation isElementValid! !

!TestOntoUMLAssociationRules methodsFor: 'running' stamp: 'volosmat 4/25/2014 19:04'!
testMemberOf
	| memberOf collective |
	collective := Collective name: 'Friends' metaModel: metaModel.
	
	memberOf := MemberOf 	name: 				'person' 
								start:				(Kind name: 'Person' metaModel: metaModel)
								startMultiplicity: 	'1'
								end: 				collective
								endMultiplicity: 		'1..*'.
	self assert: memberOf isElementValid.
	
	memberOf := MemberOf 	name: 				'alien' 
								start:				(Kind name: 'Alien' metaModel: metaModel)
								startMultiplicity: 	'1'
								end: 				(Kind name: 'TestElement' metaModel: metaModel)
								endMultiplicity: 		'1..*'.
	self deny: memberOf isElementValid.
	
	memberOf := MemberOf 	name: 				'animal' 
								start:				(Kind name: 'Animal' metaModel: metaModel)
								startMultiplicity: 	'1'
								end: 				collective
								endMultiplicity: 		'1..*'.
	collective extensional: true.
	self deny: memberOf isElementValid.! !

!TestOntoUMLAssociationRules methodsFor: 'running' stamp: 'volosmat 4/25/2014 19:07'!
testSubCollectionOf
	| subCollectionOf collective |
	collective := Collective name: 'Company' metaModel: metaModel.
	
	subCollectionOf := SubCollectionOf 	name: 				'management' 
											start:				(Collective name: 'Management' metaModel: metaModel)
											startMultiplicity: 	'1'
											end: 				collective
											endMultiplicity: 		'1..*'.
	self assert: subCollectionOf isElementValid.
	
	subCollectionOf := SubCollectionOf 	name: 				'projectTeam' 
											start:				(Kind name: 'ProjectTeam' metaModel: metaModel)
											startMultiplicity: 	'1'
											end: 				collective
											endMultiplicity: 		'1..*'.
	self deny: subCollectionOf isElementValid.
	! !

!TestOntoUMLAssociationRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 20:56'!
testSubQuantityOf
	| subQuantityOf |
	subQuantityOf := SubQuantityOf 
				name: 'SubQuantityOf'
				start: (Quantity name: 'MineralWater' metaModel: metaModel)
				startMultiplicity: '1'
				end: (Quantity name: 'Minerals' metaModel: metaModel)
				endMultiplicity: '1'.
				
	self deny: subQuantityOf isElementValid.
	
	subQuantityOf essential: true.
	
	self assert: subQuantityOf isElementValid! !

TestCase subclass: #TestOntoUMLClassRules
	instanceVariableNames: 'metaModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel-Tests'!

!TestOntoUMLClassRules methodsFor: 'initialize-release' stamp: 'volosmat 4/3/2014 17:53'!
setUp
	self 
		setUpDB! !

!TestOntoUMLClassRules methodsFor: 'private' stamp: 'volosmat 4/3/2014 17:49'!
setUpDB
	
	metaModel := MetaModel new! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:19'!
testCategory
	| category |
	category := Category
				name: 'Category'
				abstract: true
				metaModel: metaModel.
				
	self deny: category isElementValid.
	
	category
		addSubObject: (Kind name: 'SubObject' metaModel: metaModel).
		
	self assert: category isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/24/2014 23:40'!
testCollective
	| collective |
	collective := Collective name: 'Friends' metaModel: metaModel.
	
	MemberOf 	name: 				'person' 
				start:				(Kind name: 'Person' metaModel: metaModel)
				startMultiplicity: 	'0'
				end: 				collective
				endMultiplicity: 		'1..*'.
						
	self assert: collective isElementValid.
	
	MemberOf 	name: 				'animal' 
				start:				(Kind name: 'Animal' metaModel: metaModel)
				startMultiplicity: 	'0'
				end: 				collective
				endMultiplicity: 		'1..*'.
	collective extensional: true.
	
	self deny: collective isElementValid
	! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 20:26'!
testElement
	| part1 part2 whole1 whole2 |
	part1 := Kind name: 'Part1' metaModel: metaModel.
	part2 := Kind name: 'Part2' metaModel: metaModel.
	whole1 := Kind name: 'Whole1' metaModel: metaModel.
	whole2 := Kind name: 'Whole2' metaModel: metaModel.
	
	Meronymic name: 'TestMeronymic1' start: part1 startMultiplicity: '1' end: whole1 endMultiplicity: '1'.
	Meronymic name: 'TestMeronymic2' start: part2 startMultiplicity: '1' end: whole1 endMultiplicity: '1'.
	Meronymic name: 'TestMeronymic3' start: part1 startMultiplicity: '1' end: whole2 endMultiplicity: '1'.
	
	self deny: part1 isElementValid & part2 isElementValid.
	
	( part1 searchAssociation: 'TestMeronymic1' ) sharable: true.
	( part1 searchAssociation: 'TestMeronymic3' ) sharable: true.
	
	self assert: part1 isElementValid
	! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:31'!
testMixin
	| mixin |
	mixin := Mixin
				name: 'Mixin'
				abstract: true
				metaModel: metaModel.
				
	self deny: mixin isElementValid.
	
	mixin
		addObject: (Kind name: 'MixedObject' metaModel: metaModel).
	mixin
		addObject: (Phase name: 'MixedPhase' metaModel: metaModel).
		
	self assert: mixin isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:51'!
testModeAndQuality
	| mode |
	mode := Mode name: 'Mode' metaModel: metaModel.
	
	self deny: mode isElementValid.
	
	Characterization
		name: 'Characterization'
		start: mode
		end: (Kind name: 'TestKind' metaModel: metaModel).
		
	self assert: mode isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:51'!
testNonSortal
	| nonSortal |
	nonSortal := MixinClass
				name: 'OnlyForTesting'
				abstract: true
				metaModel: metaModel.
				
	self assert: nonSortal isElementValid.
	
	ParentClass
		parentClass: (Kind name: 'Parent' metaModel: metaModel)
		child: nonSortal.
		
	self deny: nonSortal isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:00'!
testPhase
	| phase |
	phase := Phase name: 'NewPhase' metaModel: metaModel.
	ParentClass
		parentClass: (Kind name: 'PhaseParent' metaModel: metaModel)
		child: phase.
	self assert: phase isElementValid.
	
	(phase searchParent: 'PhaseParent')
		disjoint: false.
		
	self deny: phase isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:51'!
testQuantity
	| quantity kind |
	quantity := Quantity name: 'TestQuantity' metaModel: metaModel.
	kind := Kind name: 'TestKind' metaModel: metaModel.
	FormalAssociation 
		name: 'containment'
		start: quantity
		startMultiplicity: '1'
		end: kind
		endMultiplicity: '1'.
		
	self assert: quantity isElementValid.
	
	MaterialAssociation
		name: 'Test'
		start: quantity
		end: kind.
		
	self deny: quantity isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:53'!
testRelator
	| relator |
	relator := Relator name: 'Relator' metaModel: metaModel.
	Mediation
		name: 'FirstMediation'
		start: relator
		end: (Role name: 'FirstRole' metaModel: metaModel).
		
	self deny: relator isElementValid.
	
	Mediation
		name: 'SecondMediation'
		start: relator
		end: (Role name: 'SecondRole' metaModel: metaModel).
		
	self assert: relator isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:19'!
testRigidMixin
	| rigidMixin |
	rigidMixin := RigidMixinClass
				name: 'OnlyForTest' 
				abstract: true
				metaModel: metaModel.
				
	self assert: rigidMixin isElementValid.
	
	ParentClass
		parentClass: (RoleMixin name: 'RoleMixin' metaModel: metaModel)
		child: rigidMixin.
		
	self deny: rigidMixin isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 18:50'!
testRigidSortal
	| rigidSortal |
	rigidSortal := Kind name: 'RigidSortal' metaModel: metaModel.
	ParentClass
		parentClass: (Mixin name: 'Mixin' metaModel: metaModel)
		child: rigidSortal.
		
	self assert: rigidSortal isElementValid.
	
	ParentClass
		parentClass: (Phase name: 'Phase' metaModel: metaModel)
		child: rigidSortal.
		
	self deny: rigidSortal isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 18:54'!
testRole
	| role |
	role := Role name: 'Role' metaModel: metaModel.
	
	self deny: role isElementValid.
	
	ParentClass
		parentClass: (Kind name: 'Person' metaModel: metaModel)
		child: role.
	Mediation
		name: 'Mediation'
		start: role
		end: (Role name: 'AnotherRole' metaModel: metaModel).
		
	self assert: role isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 19:34'!
testRoleMixin
	| roleMixin |
	roleMixin := RoleMixin
				name: 'RoleMixin'
				abstract: true
				metaModel: metaModel.
				
	self deny: roleMixin isElementValid.
	
	Mediation
		name: 'Mediation'
		start: roleMixin
		end: (Relator name: 'Relator' metaModel: metaModel).
		
	self deny: roleMixin isElementValid.
	
	roleMixin
		addRole: (Role name: 'MixedRole' metaModel: metaModel).
		
	self assert: roleMixin isElementValid! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 18:31'!
testSubKind
	| subKind |
	subKind := SubKind name: 'TestsubKind' metaModel: metaModel.
	ParentClass parentClass: ( Kind name: 'Parent' metaModel: metaModel ) child: subKind.
	
	self assert: subKind isElementValid.
	
	subKind removeParent: 'Parent'.
	
	self deny: subKind isElementValid.
	! !

!TestOntoUMLClassRules methodsFor: 'running' stamp: 'volosmat 4/3/2014 17:52'!
testSubsranceSortal
	| element |
	element := Kind name: 'TestElement' metaModel: metaModel.
	
	self assert: element isElementValid.
	
	ParentClass parentClass: ( Kind name: 'RigidSortal' metaModel: metaModel ) child: element.
	
	self deny: element isElementValid
	! !

Object subclass: #Attribute
	instanceVariableNames: 'name dataType multiplicity instance element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Attribute class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 13:43'!
name: xName dataType: xDataType multiplicity: xMultiplicity element: xElement

	^(super new initialize)
		name: 		xName;
		dataType: 	xDataType;
		multiplicity: xMultiplicity;
		element:	xElement;
		yourself! !

!Attribute class methodsFor: 'instance creation' stamp: 'volosmat 4/24/2014 11:16'!
name: xName multiplicity: xMultiplicity
	"this constructor is only for use in class DataForGenerator to create Attribute of composition. Do not use it in another class"
	^(super new initialize)
		name:		xName;
		multiplicity:	xMultiplicity;
		yourself! !

!Attribute class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 13:32'!
new 	
	^ self shouldNotImplement ! !

!Attribute methodsFor: 'comparing' stamp: 'volosmat 4/2/2014 14:31'!
= xElement
	^ self name = xElement name! !

!Attribute methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:37'!
asString
	^name , ':' , dataType , '[' , multiplicity asString , ']'! !

!Attribute methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:28'!
dataType
	^ dataType! !

!Attribute methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:28'!
dataType: anObject
	dataType := anObject! !

!Attribute methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 13:35'!
element
	^ element! !

!Attribute methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 13:52'!
element: xElement 
	element isNil & xElement notNil
		ifTrue: [element := xElement.
			element attributes add: self];
		 
		ifFalse: [self error: 'Error to setting element of this attribute. Element was set or new element is nil.']! !

!Attribute methodsFor: 'comparing' stamp: 'volosmat 4/2/2014 14:32'!
hash
	^ self name hash! !

!Attribute methodsFor: 'accessing' stamp: 'volosmat 7/16/2013 21:49'!
instance: anObject
	instance := anObject! !

!Attribute methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:38'!
isInstance
	^ instance ifNil: [ instance := true ]! !

!Attribute methodsFor: 'accessing' stamp: 'volosmat 4/24/2014 11:26'!
multiplicity
	^ multiplicity asString ifNil: [ multiplicity := '1' ]! !

!Attribute methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:38'!
multiplicity: anObject
	multiplicity := anObject! !

!Attribute methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:29'!
name
	^ name! !

!Attribute methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:28'!
name: anObject
	name := anObject! !

Object subclass: #Element
	instanceVariableNames: 'name transformed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Element methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:50'!
isElementValid
	^ true! !

!Element methodsFor: 'accessing' stamp: 'volosmat 4/14/2014 22:21'!
isTransformed
	^transformed! !

!Element methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 14:51'!
name
	^ name! !

!Element methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 14:50'!
name: anObject
	name := anObject! !

!Element methodsFor: 'accessing' stamp: 'volosmat 4/14/2014 22:21'!
transformed: anObject
	transformed := anObject! !

Element subclass: #ElementAssociation
	instanceVariableNames: 'start startMultiplicity end endMultiplicity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

ElementAssociation subclass: #DirectedBinaryAssociation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

DirectedBinaryAssociation subclass: #DependencyRelationship
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

DependencyRelationship subclass: #Characterization
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

DependencyRelationship subclass: #Derivation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Derivation methodsFor: 'comparing' stamp: 'volosmat 4/13/2014 21:11'!
= xAssociation
	^ self name = xAssociation name! !

!Derivation methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 21:26'!
end: xEnd 
"redefine end method for special type of element deriation because one end of derivation is association"
	(xEnd isKindOf: ElementClass)
		ifTrue: [super end: xEnd];
		
		ifFalse: [end := xEnd. 
			xEnd derivated: self]! !

!Derivation methodsFor: 'validating' stamp: 'volosmat 4/3/2014 21:34'!
isElementValid
	((start isMemberOf: Relator) & (startMultiplicity ~= '1')) | ((end isMemberOf: Relator) & (endMultiplicity ~= '1')) 
		ifTrue: [ ^ false ].
	
	((start isMemberOf: MaterialAssociation) & ((startMultiplicity at: 1) = $0)) | ((end isMemberOf: MaterialAssociation) & ((endMultiplicity at: 1) = $0))
		ifTrue: [ ^ false ].
		
	((start isMemberOf: MaterialAssociation) & (end isMemberOf: Relator)) | ((start isMemberOf: Relator) & (end isMemberOf: MaterialAssociation))
		ifFalse: [ ^ false ].
		
	^ super isElementValid! !

!Derivation methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 21:26'!
start: xStart
"redefine start method for special type of element deriation because one end of derivation is association"
	(xStart isKindOf: ElementClass)
		ifTrue: [super start: xStart];
		
		ifFalse: [start := xStart. 
			xStart derivated: self]! !

DependencyRelationship subclass: #Mediation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Mediation methodsFor: 'validating' stamp: 'volosmat 4/2/2014 20:54'!
isElementValid
	^ ((start isMemberOf: Relator)
		| end isMemberOf: Relator)
		& (endMultiplicity at:1) ~= $0 & (startMultiplicity at:1) ~= $0 & super isElementValid ! !

DirectedBinaryAssociation subclass: #Meronymic
	instanceVariableNames: 'essential inseparable immutablePart immutableWhole sharable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

Meronymic subclass: #ComponentOf
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!ComponentOf methodsFor: 'validating' stamp: 'volosmat 4/25/2014 19:16'!
isElementValid
	
	
	^ (self isFunctionalWhole: start) & (self isFunctionalWhole: end)! !

!ComponentOf methodsFor: 'private' stamp: 'volosmat 4/25/2014 19:26'!
isFunctionalWhole: xElement

	(xElement isKindOf: SortalClass)
		ifTrue: [(xElement isMemberOf: Kind) | (xElement isMemberOf: SubKind)
					ifTrue: [^ super isElementValid ]].
	
	^ false! !

Meronymic subclass: #MemberOf
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!MemberOf methodsFor: 'validating' stamp: 'volosmat 4/25/2014 19:04'!
isElementValid
	( end isMemberOf: Collective )
		ifFalse: [ ^ false ].
	( end isExtensional & self isEssential not ) 
		ifTrue: [ ^ false ].
	
	^ super isElementValid! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:50'!
essential: anObject
	essential := anObject! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:50'!
immutablePart: anObject
	immutablePart := anObject! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:51'!
immutableWhole: anObject
	immutableWhole := anObject! !

!Meronymic methodsFor: 'initialize-release' stamp: 'volosmat 4/3/2014 20:35'!
initialize
	super initialize.

	essential 			:= false.
	immutablePart 		:= false.
	immutableWhole 	:= false.
	inseparable 		:= false.
	sharable			:= false.

	^self! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:51'!
inseparable: anObject
	inseparable := anObject! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:50'!
isEssential
	^ essential! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:50'!
isImmutablePart
	^ immutablePart! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:51'!
isImmutableWhole
	^ immutableWhole! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:51'!
isInseparable
	^ inseparable! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:52'!
isSharable
	^ sharable! !

!Meronymic methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:52'!
sharable: xShare
	sharable := xShare! !

Meronymic subclass: #SubCollectionOf
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!SubCollectionOf methodsFor: 'validating' stamp: 'volosmat 4/3/2014 11:27'!
isElementValid
	^ startMultiplicity = '1'
		& (start isMemberOf: Collective)
		& (end isMemberOf: Collective) & super isElementValid! !

Meronymic subclass: #SubQuantityOf
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!SubQuantityOf methodsFor: 'validating' stamp: 'volosmat 4/3/2014 11:23'!
isElementValid
	^ self isSharable = false & self isEssential = true & startMultiplicity = '1'
		& (start isMemberOf: Quantity)
		& (end isMemberOf: Quantity) & super isElementValid ! !

!ElementAssociation class methodsFor: 'instance creation' stamp: 'volosmat 3/12/2014 11:35'!
name: xName

	^(super new initialize)
		name: xName;
		yourself! !

!ElementAssociation class methodsFor: 'instance creation' stamp: 'volosmat 3/12/2014 11:35'!
name: xName start: xStart end: xEnd

	^(super new initialize)
		name: 	xName ;
		start: 	xStart;
		end: 	xEnd;
		yourself! !

!ElementAssociation class methodsFor: 'instance creation' stamp: 'volosmat 3/12/2014 11:35'!
name: xName start: xStart startMultiplicity: xStartMultiplicity end: xEnd endMultiplicity: xEndMultiplicity

	^(super new initialize)
		name: 				xName;
		start: 				xStart;
		startMultiplicity: 	xStartMultiplicity;
		end: 				xEnd;
		endMultiplicity: 		xEndMultiplicity;
		yourself! !

!ElementAssociation methodsFor: 'comparing' stamp: 'volosmat 4/13/2014 21:02'!
= xAssociation
	^ self name = xAssociation name & ( (self start = xAssociation start) | (self start = xAssociation end) ) & ( self end = xAssociation start | self end = xAssociation end )! !

!ElementAssociation methodsFor: 'private' stamp: 'volosmat 3/12/2014 11:28'!
asAssociation: xAssociation 
	xAssociation start: start.
	xAssociation end: end.
	xAssociation endMultiplicity: endQuantity.
	xAssociation startMultiplicity: startQuantity.
	start removeAssociation: name.
	end removeAssociation: name.
	start associations add: xAssociation.
	end associations add: xAssociation. 
	
	^ xAssociation! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:11'!
asCharacterization
	^ self asAssociation: ( Characterization name: name )! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:12'!
asComponentOf
	^ self asAssociation: ( ComponentOf name: name )! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:10'!
asDerivation
	^ self asAssociation: ( Derivation name: name )! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:12'!
asFormalAssociation
	^ self asAssociation: ( FormalAssociation name: name )! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:13'!
asMaterialAssociation
	^ self asAssociation: ( MaterialAssociation name: name )! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:11'!
asMediation
	^ self asAssociation: ( Mediation name: name )! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:12'!
asMemberOf
	^ self asAssociation: ( MemberOf name: name )! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:12'!
asSubCollectionOf
	^ self asAssociation: ( SubCollectionOf name: name )! !

!ElementAssociation methodsFor: 'converting' stamp: 'volosmat 12/30/2013 13:12'!
asSubQuantityOf
	^ self asAssociation: ( SubQuantityOf name: name )! !

!ElementAssociation methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 11:30'!
end
	^ end! !

!ElementAssociation methodsFor: 'accessing' stamp: 'volosmat 4/27/2014 15:39'!
end: xEnd 
	[xEnd searchAssociation: name]
		on: Error
		do: [end isNil & xEnd notNil
				ifTrue: [end := xEnd.
					end associations add: self.
					^ self]; 
				
				ifFalse: [self error: 'Is not allowed to change end of association.']].
	
	self error: 'Association ',self name,' exist in this element.'! !

!ElementAssociation methodsFor: 'accessing' stamp: 'volosmat 4/24/2014 11:25'!
endMultiplicity
	^ endMultiplicity asString! !

!ElementAssociation methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:28'!
endMultiplicity: anObject
	endMultiplicity := anObject! !

!ElementAssociation methodsFor: 'comparing' stamp: 'volosmat 4/2/2014 16:40'!
hash
	^ self name hash & self start hash & self end hash! !

!ElementAssociation methodsFor: 'initialize-release' stamp: 'volosmat 4/2/2014 15:52'!
initialize

	super initialize.

	transformed		:= false.
	startMultiplicity 	:= '0'.
	endMultiplicity 		:= '0'.
	^self! !

!ElementAssociation methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 11:30'!
start
	^ start! !

!ElementAssociation methodsFor: 'accessing' stamp: 'volosmat 4/27/2014 15:38'!
start: xStart 
	[xStart searchAssociation: name]
		on: Error
		do: [ start isNil & xStart notNil
		ifTrue: [start := xStart.
			start associations add: self.
			^ self];
		
		ifFalse: [self error: 'Is not allowed to change start of association.'] ].
	
	self error: 'Association ',self name,' exist in this element.'! !

!ElementAssociation methodsFor: 'accessing' stamp: 'volosmat 4/24/2014 11:25'!
startMultiplicity
	^ startMultiplicity asString! !

!ElementAssociation methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:29'!
startMultiplicity: anObject
	startMultiplicity := anObject! !

ElementAssociation subclass: #FormalAssociation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

ElementAssociation subclass: #MaterialAssociation
	instanceVariableNames: 'derivated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!MaterialAssociation methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 09:27'!
derivated
	^ derivated! !

!MaterialAssociation methodsFor: 'accessing' stamp: 'volosmat 4/27/2014 15:42'!
derivated: anObject
	derivated isNil 
		ifTrue: [ derivated := anObject ];
		ifFalse: [ self error: 'Is not allowed to change derivated of this association.' ]
	! !

!MaterialAssociation methodsFor: 'validating' stamp: 'volosmat 4/3/2014 21:46'!
isElementValid
	^ (derivated isMemberOf: Derivation) & ((startMultiplicity at:1) ~= $0) & ((endMultiplicity at:1) ~= $0) & super isElementValid ! !

Element subclass: #ElementClass
	instanceVariableNames: 'methods associations parents abstract attributes metaModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!ElementClass class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 15:31'!
name: xName abstract: xAbstract metaModel: xMetaModel

	^ (super new initialize)
		name: 			xName;
		abstract: 		xAbstract;
		metaModel: 	xMetaModel;
		yourself! !

!ElementClass class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 15:31'!
name: xName metaModel: xMetaModel

	^ (super new initialize)
		name: 			xName;
		metaModel: 	xMetaModel;
		yourself! !

!ElementClass methodsFor: 'comparing' stamp: 'volosmat 4/2/2014 00:55'!
= xElement
	^ self name = xElement name! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 7/16/2013 14:14'!
abstract: anObject
	abstract := anObject! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:40'!
asCategory
	^ self asElement: ( Category name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:40'!
asCollective
	^ self asElement: ( Collective name: name ).! !

!ElementClass methodsFor: 'private' stamp: 'volosmat 4/25/2014 18:56'!
asElement: xElement 
	"xElement abstract: abstract.
	xElement methods: methods.
	xElement attributes: attributes.
	xElement parent removeAll. 
	xElement associations: associations.
	associations
		do: [:each | each start name = name
				ifTrue: [each start: xElement];
				
				ifFalse: [each end: xElement]].
	parents
		do: [:each | xElement parent: each].
	^ xElement"! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 00:19'!
asKind
	^ self asElement: ( Kind name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:40'!
asMixin
	^ self asElement: ( Mixin name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:39'!
asMode
	^ self asElement: ( Mode name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:39'!
asPhase
	^ self asElement: ( Phase name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:39'!
asQuality
	^ self asElement: ( Quality name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:39'!
asQuantity
	^ self asElement: ( Quantity name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:40'!
asRelator
	^ self asElement: ( Relator name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:38'!
asRole
	^ self asElement: ( Role name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:40'!
asRoleMixin
	^ self asElement: ( RoleMixin name: name ).! !

!ElementClass methodsFor: 'converting' stamp: 'volosmat 12/26/2013 18:38'!
asSubKind
	^ self asElement: ( SubKind name: name ).! !

!ElementClass methodsFor: 'accessing' stamp: 'Anonymous 4/2/2013 23:17'!
associations
	^ associations! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 14:35'!
associations: anObject
	associations := anObject.
	
	associations ifNil: [ associations := Set new ]! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 12/11/2013 23:57'!
attributes
	^ attributes! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 14:34'!
attributes: anObject
	attributes := anObject.
	
	attributes ifNil: [ attributes := Set new ]! !

!ElementClass methodsFor: 'comparing' stamp: 'volosmat 4/2/2014 00:53'!
hash
	^ self name hash! !

!ElementClass methodsFor: 'initialize-release' stamp: 'volosmat 4/2/2014 14:28'!
initialize
	super initialize.

	name 			:= nil.
	attributes 		:= Set new.
	methods 		:= Set new.
	associations 	:= Set new.
	parents 		:= Set new.
	transformed 	:= false.
	abstract 		:= false.
	
	^ self! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 12:28'!
isAbstract
	^ abstract ifNil: [ abstract := false ]! !

!ElementClass methodsFor: 'validating' stamp: 'volosmat 4/25/2014 18:55'!
isElementNameValid
	| allowed |
	
	allowed := ($0 to: $9), {$_}, ($A to: $Z), ($a to: $z).
	(name detect: [:c | (allowed includes: c) not] ifNone: [ ])
		ifNotNil: [ ^ false].
	name first canBeGlobalVarInitial ifFalse:[^false].
	^ true! !

!ElementClass methodsFor: 'validating' stamp: 'volosmat 4/27/2014 20:43'!
isElementValid
	| meronymic |
	"if is attribute isSharable in meronymic association true the part can be join to more than one whole"
	meronymic := associations select: [ :each | (each isKindOf: Meronymic) & (each start = self) ].
	
	( meronymic anySatisfy: [ :each | each isSharable = false ] ) & (meronymic size > 1)
		ifTrue: [ ^ false ].
		
	^ super isElementValid! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 15:02'!
metaModel
	^ metaModel! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 15:04'!
metaModel: xMetaModel 
	metaModel isNil & xMetaModel notNil
		ifTrue: [metaModel := xMetaModel. 
			metaModel addElement: self];
		
		ifFalse: [self error: 'Error to adding this element to MetaModel. MetaModel is nill or this element is in another MetaModel']! !

!ElementClass methodsFor: 'accessing' stamp: 'Anonymous 4/2/2013 23:18'!
methods
	^ methods! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 14:34'!
methods: anObject
	methods := anObject.
	
	methods ifNil: [ methods := Set new ]! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 10:51'!
parents
	^ parents! !

!ElementClass methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 14:34'!
parents: anObject
	parents := anObject.
	
	parents ifNil: [ parents := Set new ]! !

!ElementClass methodsFor: 'removing' stamp: 'volosmat 4/2/2014 14:44'!
removeAssociation: xName
	associations := associations reject: [ :each | each name = xName ]! !

!ElementClass methodsFor: 'removing' stamp: 'volosmat 4/2/2014 14:46'!
removeAttribute: xName
	attributes := attributes reject: [ :each | each name = xName ]! !

!ElementClass methodsFor: 'removing' stamp: 'volosmat 11/27/2013 17:53'!
removeMethod: xName
	methods := methods reject: [ :each | each name = xName ]! !

!ElementClass methodsFor: 'removing' stamp: 'volosmat 4/3/2014 18:32'!
removeParent: xName
	parents := parents reject: [ :each | each parentClass name = xName ]! !

!ElementClass methodsFor: 'search' stamp: 'volosmat 12/17/2013 18:00'!
searchAssociation: xName 
	^ associations detect: [ :each | each name = xName ].! !

!ElementClass methodsFor: 'search' stamp: 'volosmat 12/11/2013 22:11'!
searchAttribute: xName 
	^ attributes detect: [ :each | each name = xName ].! !

!ElementClass methodsFor: 'search' stamp: 'volosmat 11/27/2013 01:03'!
searchMethod: xName 
	^ methods detect: [ :each | each name = xName ].! !

!ElementClass methodsFor: 'search' stamp: 'volosmat 4/3/2014 18:59'!
searchParent: xName 
	^ parents detect: [ :each | each parentClass name = xName ].! !

ElementClass subclass: #MomentClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

MomentClass subclass: #Mode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Mode methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:53'!
isElementValid
	^ (associations
		allSatisfy: [:each | each isMemberOf: Characterization])
		& (associations size > 0) & super isElementValid ! !

!MomentClass methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:53'!
isElementValid
	^ super isElementValid! !

MomentClass subclass: #Quality
	instanceVariableNames: 'asAttribute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Quality methodsFor: 'accessing' stamp: 'Anonymous 4/8/2013 09:51'!
asAttribute
	^ asAttribute! !

!Quality methodsFor: 'accessing' stamp: 'Anonymous 4/8/2013 09:51'!
asAttribute: anObject
	asAttribute := anObject! !

!Quality methodsFor: 'initialize-release' stamp: 'volosmat 3/13/2014 11:46'!
initialize
	super initialize.
	
	asAttribute := false.
	
	^self! !

!Quality methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:53'!
isElementValid
	^ (associations
		allSatisfy: [:each | each isMemberOf: Characterization])
		& (associations size > 0) & super isElementValid! !

MomentClass subclass: #Relator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Relator methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:53'!
isElementValid
	^ (associations 
		select: [:each | each isMemberOf: Mediation]) size > 1 & super isElementValid! !

ElementClass subclass: #ObjectClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

ObjectClass subclass: #MixinClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!MixinClass methodsFor: 'validating' stamp: 'volosmat 4/3/2014 19:43'!
isElementValid
	^ (parents
		anySatisfy: [:each | each parentClass isKindOf: SortalClass]) not & super isElementValid ! !

MixinClass subclass: #NonRigidMixinClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

NonRigidMixinClass subclass: #AntiRigidMixinClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

AntiRigidMixinClass subclass: #RoleMixin
	instanceVariableNames: 'roles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!RoleMixin methodsFor: 'adding' stamp: 'volosmat 4/3/2014 19:33'!
addRole: xRole 
	(xRole isMemberOf: Role) 
		ifTrue: [roles add: xRole.
			ParentClass parentClass: self child: xRole];
		
		ifFalse: [self error: 'We can add only role to collection.']! !

!RoleMixin methodsFor: 'initialize-release' stamp: 'volosmat 4/2/2014 19:57'!
initialize

	super initialize.

	roles := Set new.

	^self! !

!RoleMixin methodsFor: 'validating' stamp: 'volosmat 4/3/2014 19:33'!
isElementValid
	^ (associations 
		anySatisfy: [:each | each isMemberOf: Mediation])
		& (roles size > 0) & super isElementValid! !

!RoleMixin methodsFor: 'removing' stamp: 'volosmat 4/2/2014 19:57'!
removeRole: xName
	| roleToRemove |
	
	roleToRemove := roles detect: [ :each | each name = xName ].
	roleToRemove removeParent: self name.
	roles remove: roleToRemove ifAbsent: [].! !

!RoleMixin methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 19:56'!
roles
	^ roles copy! !

NonRigidMixinClass subclass: #SemiRigidMixinClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

SemiRigidMixinClass subclass: #Mixin
	instanceVariableNames: 'mixObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Mixin methodsFor: 'adding' stamp: 'volosmat 4/2/2014 20:01'!
addObject: xObject
	mixObjects add: xObject.
	ParentClass parentClass: self child: xObject! !

!Mixin methodsFor: 'initialize-release' stamp: 'volosmat 4/2/2014 19:58'!
initialize

	super initialize.
	
	mixObjects := Set new.
	
	^self! !

!Mixin methodsFor: 'validating' stamp: 'volosmat 4/2/2014 20:09'!
isElementValid
	[mixObjects
		detect: [:each | each isKindOf: RigidSortalClass]]
		on: Error
		do: [^ false].
	[mixObjects
		detect: [:each | each isKindOf: AntiRigidSortalClass]]
		on: Error
		do: [^ false].
	^ mixObjects size = 2 & super isElementValid! !

!Mixin methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 20:00'!
mixObjects
	^ mixObjects copy! !

!Mixin methodsFor: 'adding' stamp: 'volosmat 4/2/2014 20:02'!
removeObject: xName
	| objectToRemove |
	
	objectToRemove := mixObjects detect: [ :each | each name = xName ].
	objectToRemove removeParent: self name.
	mixObjects remove: objectToRemove ifAbsent: [].! !

!SemiRigidMixinClass methodsFor: 'validating' stamp: 'volosmat 4/3/2014 19:25'!
isElementValid
	^ (parents 
		anySatisfy: [:each | each parentClass isKindOf: AntiRigidMixinClass]) not & super isElementValid! !

MixinClass subclass: #RigidMixinClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

RigidMixinClass subclass: #Category
	instanceVariableNames: 'subObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Category methodsFor: 'adding' stamp: 'volosmat 4/3/2014 19:12'!
addSubObject: xObject 
	(xObject isKindOf: SortalClass)
		ifTrue: [subObjects add: xObject.
			ParentClass parentClass: self complete: false disjoint: true child: xObject]; 
		
		ifFalse: [self error: 'We can add only Sortal Class to collection.']! !

!Category methodsFor: 'initialize-release' stamp: 'volosmat 4/2/2014 18:00'!
initialize

	super initialize.

	subObjects := Set new.

	^self! !

!Category methodsFor: 'validating' stamp: 'volosmat 4/3/2014 19:13'!
isElementValid
	^ (subObjects
		allSatisfy: [:each | each isKindOf: SortalClass]) 
		& subObjects size > 0 & super isElementValid! !

!Category methodsFor: 'adding' stamp: 'volosmat 4/2/2014 18:20'!
removeSubObject: xName
	| objectToRemove |
	
	objectToRemove := subObjects detect: [ :each | each name = xName ].
	objectToRemove removeParent: self name.
	subObjects remove: objectToRemove ifAbsent: [].! !

!Category methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 18:04'!
subObjects
	^ subObjects copy! !

!RigidMixinClass methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:53'!
isElementValid
	^ (parents 
		anySatisfy: [:each | each parentClass isKindOf: AntiRigidMixinClass]) not & super isElementValid! !

!ObjectClass methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:53'!
isElementValid
	^ super isElementValid! !

ObjectClass subclass: #SortalClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

SortalClass subclass: #AntiRigidSortalClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

AntiRigidSortalClass subclass: #Phase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Phase methodsFor: 'validating' stamp: 'volosmat 4/12/2014 15:52'!
isElementValid
	^ (parents anySatisfy: [:each | each isDisjoint & each isComplete])
			& super isElementValid ! !

AntiRigidSortalClass subclass: #Role
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Role methodsFor: 'validating' stamp: 'volosmat 4/3/2014 18:56'!
isElementValid
	^ (parents
		anySatisfy: [:each | each parentClass isKindOf: RigidSortalClass])
		& (parents size > 0) & ( associations anySatisfy: [:each | each isMemberOf: Mediation] )
			& super isElementValid! !

SortalClass subclass: #RigidSortalClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!RigidSortalClass methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:53'!
isElementValid
	^ (parents
		anySatisfy: [:each | (each parentClass isKindOf: AntiRigidSortalClass) 
				| ( each parentClass isKindOf: AntiRigidMixinClass)]) not & super isElementValid! !

RigidSortalClass subclass: #SubKind
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!SubKind methodsFor: 'validating' stamp: 'volosmat 4/2/2014 14:53'!
isElementValid
	parents do: [:each | 
		( each parentClass isKindOf: SubstanceSortal ) ifTrue: [ ^ super isElementValid ].
		( each parentClass isMemberOf: SubKind ) ifTrue: [ ^ each parentClass isElementValid ] ].

     ^ false.! !

RigidSortalClass subclass: #SubstanceSortal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

SubstanceSortal subclass: #Collective
	instanceVariableNames: 'extensional'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Collective methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 17:34'!
extensional: anObject
	extensional := anObject! !

!Collective methodsFor: 'validating' stamp: 'volosmat 4/24/2014 23:43'!
isElementValid
	| parts |
	self isExtensional
		ifTrue: [ parts := associations select: [ :each | (each isMemberOf: MemberOf) | (each isMemberOf: SubCollectionOf) ] ];
		ifFalse: [ ^ super isElementValid ].
		
	^ (parts allSatisfy: [ :each | each startMultiplicity ~= '0' ]) & super isElementValid 
	
     ! !

!Collective methodsFor: 'accessing' stamp: 'volosmat 4/24/2014 23:39'!
isExtensional
	^ extensional ifNil: [ extensional := false ]! !

SubstanceSortal subclass: #Kind
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Kind methodsFor: 'validating' stamp: 'volosmat 4/10/2014 19:11'!
isElementValid
	^ (parents
		anySatisfy: [:each | each parentClass isKindOf: MomentClass]) not & super isElementValid ! !

SubstanceSortal subclass: #Quantity
	instanceVariableNames: 'isExtensional'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Quantity methodsFor: 'validating' stamp: 'volosmat 4/3/2014 18:28'!
isElementValid
	^ (associations allSatisfy: [:each | (each isMemberOf: FormalAssociation)
			| (each isMemberOf: SubQuantityOf)])
			& super isElementValid! !

!SubstanceSortal methodsFor: 'validating' stamp: 'volosmat 4/2/2014 17:21'!
isElementValid
	^ (parents
		anySatisfy: [:each | each parentClass isKindOf: RigidSortalClass]) not & super isElementValid ! !

Object subclass: #MetaModel
	instanceVariableNames: 'modelElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!MetaModel class methodsFor: 'search' stamp: 'volosmat 4/1/2014 16:52'!
checkIfObject: xObject isInCollection: xCollection

	^ xCollection anySatisfy: [ :each | each name = xObject name ]! !

!MetaModel methodsFor: 'adding' stamp: 'volosmat 4/27/2014 13:11'!
addElement: xElement
	[ modelElements at: xElement name ]
		on: Error
		do: [ modelElements at: xElement name put: xElement ]! !

!MetaModel methodsFor: 'accessing' stamp: 'volosmat 4/27/2014 13:38'!
at: xName
	"Return element with name xName from collection of modelElements"
	^ (modelElements at: xName) copy! !

!MetaModel methodsFor: 'initialize-release' stamp: 'volosmat 4/27/2014 13:07'!
initialize

	super initialize.
	
	modelElements := KeyedTree new.
	
	^self! !

!MetaModel methodsFor: 'accessing' stamp: 'volosmat 10/28/2013 22:09'!
modelElements
	^ modelElements copy! !

!MetaModel methodsFor: 'removing' stamp: 'volosmat 4/27/2014 13:38'!
removeElement: xElement
	| collection |
	collection := OrderedCollection new.
	collection add: xElement name.
	
	modelElements removePath: collection
! !

Object subclass: #Method
	instanceVariableNames: 'name returnDataType parameters abstract instance element constructor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 14:00'!
name: xName abstract: xAbstract element: xElement

   ^(super new initialize)
      name: 		xName;
	abstract: 	xAbstract;
	element:	xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 14:00'!
name: xName abstract: xAbstract instance: xInstance element: xElement

   ^(super new initialize)
      name: 		xName;
	abstract: 	xAbstract;
	instance: 	xInstance;
	element:	xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 14:00'!
name: xName element: xElement

   ^(super new initialize)
      name: 		xName;
	element: 	xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/15/2014 09:56'!
name: xName instance: xInstance constructor: xConstructor element: xElement

   ^(super new initialize)
      name: 			xName;
	instance: 		xInstance;
	constructor: 	xConstructor;
	element: 		xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 14:01'!
name: xName instance: xInstance element: xElement

   ^(super new initialize)
      name: 		xName;
	instance: 	xInstance;
	element: 	xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 14:02'!
name: xName returnDataType: xReturnDataType abstract: xAbstract element: xElement

   ^(super new initialize)
      name: 				xName;
      returnDataType: 	xReturnDataType;
	abstract: 			xAbstract;
	element:			xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 14:02'!
name: xName returnDataType: xReturnDataType abstract: xAbstract instance: xInstance element: xElement

   ^(super new initialize)
      name: 				xName;
      returnDataType: 	xReturnDataType;
	abstract: 			xAbstract;
	instance: 			xInstance;
	element:			xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 14:01'!
name: xName returnDataType: xReturnDataType element: xElement

   ^(super new initialize)
      name: 				xName;
      returnDataType: 	xReturnDataType;
	element: 			xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 14:03'!
name: xName returnDataType: xReturnDataType instance: xInstance element: xElement

   ^(super new initialize)
      name: 				xName;
      returnDataType: 	xReturnDataType;
	instance: 			xInstance;
	element:			xElement;
      yourself! !

!Method class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 13:51'!
new
	^ self shouldNotImplement ! !

!Method methodsFor: 'comparing' stamp: 'volosmat 4/2/2014 14:32'!
= xElement
	^ self name = xElement name & self parameters = xElement parameters! !

!Method methodsFor: 'accessing' stamp: 'volosmat 7/16/2013 14:04'!
abstract: anObject
	abstract := anObject! !

!Method methodsFor: 'accessing' stamp: 'RostislavNovak 12/18/2013 00:24'!
asString
	^name , '(' , ( parameters inject: '' into: [ :str :a | str , a , ',' ]) , ')' , ':' , returnDataType! !

!Method methodsFor: 'accessing' stamp: 'volosmat 4/15/2014 09:50'!
constructor: anObject
	constructor := anObject! !

!Method methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 13:56'!
element
	^ element! !

!Method methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 13:58'!
element: xElement 
	element isNil & xElement notNil
		ifTrue: [element := xElement.
			element methods add: self]; 
		
		ifFalse: [self error: 'Error to setting element of this method. Element was set or new element is nil.']! !

!Method methodsFor: 'comparing' stamp: 'volosmat 4/2/2014 16:39'!
hash
	^ self name hash  & self parameters hash! !

!Method methodsFor: 'initialize-release' stamp: 'volosmat 4/16/2014 23:16'!
initialize
	super initialize.
	
	name 				:= nil.
	parameters 		:= OrderedCollection new.
	returnDataType	:= nil.
	abstract 			:= false.
	instance 			:= true.
	constructor			:= false.

	^self! !

!Method methodsFor: 'accessing' stamp: 'volosmat 7/16/2013 14:04'!
instance: anObject
	instance := anObject! !

!Method methodsFor: 'accessing' stamp: 'volosmat 4/15/2014 09:51'!
isAbstract
	^abstract! !

!Method methodsFor: 'accessing' stamp: 'volosmat 4/15/2014 09:51'!
isConstructor
	^ constructor! !

!Method methodsFor: 'accessing' stamp: 'volosmat 4/15/2014 09:51'!
isInstance
	^ instance! !

!Method methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:25'!
name
	^ name! !

!Method methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:25'!
name: anObject
	name := anObject! !

!Method methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:25'!
parameters
	^ parameters! !

!Method methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:25'!
parameters: anObject
	parameters := anObject! !

!Method methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:25'!
returnDataType
	^ returnDataType! !

!Method methodsFor: 'accessing' stamp: 'Anonymous 4/5/2013 10:25'!
returnDataType: anObject
	returnDataType := anObject! !

Object subclass: #ParentClass
	instanceVariableNames: 'disjoint complete parentClass child'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GMF-MetaModel'!

!ParentClass class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 18:15'!
new
	^ self shouldNotImplement ! !

!ParentClass class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 18:15'!
parentClass: xParentClass child: xChild

	^ ( super new initialize )
		parentClass: xParentClass;
		child: xChild;
		yourself! !

!ParentClass class methodsFor: 'instance creation' stamp: 'volosmat 4/2/2014 18:15'!
parentClass: xParentClass complete: xComplete disjoint: xDisjoint child: xChild
 
	^ super new initialize 
		 parentClass: xParentClass;
		 complete: xComplete;
		 disjoint: xDisjoint;
		 child: xChild;
		 yourself! !

!ParentClass methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 18:12'!
child 
	^ child ! !

!ParentClass methodsFor: 'accessing' stamp: 'volosmat 4/2/2014 18:14'!
child: xChild 
	child isNil & xChild notNil
		ifTrue: [child := xChild.
			xChild parents add: self]; 
		
		ifFalse: [self error: 'Error to setting child of this parent. New child is nil or child was set.']! !

!ParentClass methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:21'!
complete: xComplete
	complete := xComplete! !

!ParentClass methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:22'!
disjoint: xDisjoint
	disjoint := xDisjoint! !

!ParentClass methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 19:01'!
isComplete
	^ complete ifNil: [ complete := true ]
	! !

!ParentClass methodsFor: 'accessing' stamp: 'volosmat 4/3/2014 19:01'!
isDisjoint
	^ disjoint ifNil: [ disjoint := true ]! !

!ParentClass methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:20'!
parentClass
	^ parentClass! !

!ParentClass methodsFor: 'accessing' stamp: 'volosmat 3/12/2014 11:20'!
parentClass: anObject
	parentClass := anObject! !
